/** @file
 * @brief Config class header. This class represents the actual configuration.
 *
 *
 *
 * @copyright Intra2net AG
 * @license GPLv2
*/

#ifndef CONFIG_H
#define CONFIG_H

#include <boost/program_options.hpp>
#include <boost/foreach.hpp>
#include <boost/filesystem.hpp>
#include <boost/regex.hpp>
#include <boost/shared_ptr.hpp>

#include <string>
#include <iostream>
#include <fstream>

#include "service.h"
#include "logger.h"

#include "dhs.h"
#include "ods.h"

namespace po = boost::program_options;
namespace fs = boost::filesystem;

using namespace std;

typedef boost::shared_ptr<Service> ServicePtr;
typedef boost::shared_ptr<Logger> LoggerPtr;

class Config
{

private:

    po::options_description *Opt_desc_cmd;
    po::options_description *Opt_desc_conf_main;
    po::options_description *Opt_desc_conf_service;
    po::variables_map Variables_map;

    list<ServicePtr> Services;
    LoggerPtr Log;

    bool Daemon_mode;
    string Logfile;
    int Loglevel;
    bool Syslog;
    string Config_path;

    ServicePtr create_service(const string&,const string&,const string&,const string&);
    int load_main_config_file(const string&);
    int load_service_config_file(const string&);

public:

    Config(LoggerPtr);

    ~Config();

    int parse_cmd_line(int, char **);

    int load_config_from_files();

    list<ServicePtr> get_services();

    po::options_description* get_opt_desc_cmd();

    po::options_description* get_opt_desc_conf_main();

    po::options_description* get_opt_desc_conf_service();

    int get_loglevel();

    bool get_daemon_mode();

    void delete_services();

};

#endif
